@@ -1,5 +1,5 @@
 /* cli.c -- Command line interface for GNU Mailutils
-   Copyright (C) 2016-2017 Free Software Foundation, Inc.
+   Copyright (C) 2016-2020 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -55,7 +55,7 @@ const char mu_version_copyright[] =
   /* Do *not* mark this string for translation.  %s is a copyright
      symbol suitable for this locale, and %d is the copyright
      year.  */
-  "Copyright %s 2007-2017 Free Software Foundation, inc.";
+  "Copyright %s 2007-2019 Free Software Foundation, inc.";
 
 void
 mu_version_hook (struct mu_parseopt *po, mu_stream_t stream)
@@ -252,23 +252,23 @@ static struct mu_option mu_no_config_option = {
 
 /* These options are always available for utilities that use at least
    one of default configuration files */
-static struct mu_option mu_config_options[] = {
-  { "config-file",    0,  N_("FILE"),  MU_OPTION_IMMEDIATE,
-    N_("load this configuration file; implies --no-config"),
-    mu_c_string, NULL, config_file },
-  
+static struct mu_option mu_config_lint_options[] = {
   { "config-verbose", 0,  NULL,        MU_OPTION_IMMEDIATE,
     N_("verbosely log parsing of the configuration files"),
     mu_c_string, NULL, config_verbose },
-  
   { "config-lint",    0,  NULL,        MU_OPTION_IMMEDIATE,
     N_("check configuration file syntax and exit"),
     mu_c_string, NULL, config_lint },
+  MU_OPTION_END  
+};
 
+static struct mu_option mu_config_override_options[] = {
+  { "config-file",    0,  N_("FILE"),  MU_OPTION_IMMEDIATE,
+    N_("load this configuration file; implies --no-config"),
+    mu_c_string, NULL, config_file },
   { "set",            0,  N_("PARAM=VALUE"), MU_OPTION_IMMEDIATE,
     N_("set configuration parameter"),
     mu_c_string, NULL, param_set },
-
   MU_OPTION_END  
 };
 
@@ -425,29 +425,33 @@ init_options (mu_opool_t pool,
   if (hints->flags & CONFIG_ENABLED)
     {
       opool_add_option (pool, &mu_config_option_header);
-      opool_add_options (pool, mu_config_options);
-      if (hints->flags & MU_CFHINT_SITE_FILE)
+      opool_add_options (pool, mu_config_lint_options);
+      if (!(hints->flags & MU_CFHINT_NO_CONFIG_OVERRIDE))
 	{
-	  opool_add_options (pool, mu_site_config_options);
-	  if (hints->flags & MU_CFHINT_PER_USER_FILE)
+	  opool_add_options (pool, mu_config_override_options);
+	  if (hints->flags & MU_CFHINT_SITE_FILE)
 	    {
-	      opool_add_options (pool, mu_user_config_options);
-	      opool_add_option (pool, &mu_no_config_option);
+	      opool_add_options (pool, mu_site_config_options);
+	      if (hints->flags & MU_CFHINT_PER_USER_FILE)
+		{
+		  opool_add_options (pool, mu_user_config_options);
+		  opool_add_option (pool, &mu_no_config_option);
+		}
+	      else
+		{
+		  struct mu_option opt = mu_no_config_option;
+		  opt.opt_flags = MU_OPTION_ALIAS;
+		  opool_add_option (pool, &opt);
+		}
 	    }
-	  else
+	  else if (hints->flags & MU_CFHINT_PER_USER_FILE)
 	    {
 	      struct mu_option opt = mu_no_config_option;
+	      opool_add_options (pool, mu_user_config_options);
 	      opt.opt_flags = MU_OPTION_ALIAS;
 	      opool_add_option (pool, &opt);
 	    }
 	}
-      else if (hints->flags & MU_CFHINT_PER_USER_FILE)
-	{
-	  struct mu_option opt = mu_no_config_option;
-	  opool_add_options (pool, mu_user_config_options);
-	  opt.opt_flags = MU_OPTION_ALIAS;
-	  opool_add_option (pool, &opt);
-	}
       mu_list_append (oplist, opool_end_option (pool));
     }
   
@@ -483,7 +487,9 @@ run_commit (void *item, void *data)
     | MU_PARSEOPT_BUG_ADDRESS				\
     | MU_PARSEOPT_EXTRA_INFO				\
     | MU_PARSEOPT_VERSION_HOOK				\
-    | MU_PARSEOPT_NEGATION)
+    | MU_PARSEOPT_PROG_NAME   				\
+    | MU_PARSEOPT_NEGATION                              \
+    | MU_PARSEOPT_SPECIAL_ARGS )
 
 void
 mu_cli_ext (int argc, char **argv,
@@ -520,8 +526,13 @@ mu_cli_ext (int argc, char **argv,
   /* Set program name */
   if (!(hints.flags & MU_CFHINT_PROGRAM))
     {
-      mu_set_program_name (argv[0]);
-      hints.program = (char*) mu_program_name;
+      if (pohint->po_flags & MU_PARSEOPT_PROG_NAME)
+	hints.program = (char *) pohint->po_prog_name;
+      else
+	{
+	  mu_set_program_name (argv[0]);
+	  hints.program = (char*) mu_program_name;
+	}
       hints.flags |= MU_CFHINT_PROGRAM;
     }
 
@@ -581,7 +592,12 @@ mu_cli_ext (int argc, char **argv,
       po.po_help_hook = extra_help_hook;
       flags |= MU_PARSEOPT_HELP_HOOK;
     }
-
+  else if (pohint->po_flags & MU_PARSEOPT_HELP_HOOK)
+    {
+      po.po_help_hook = pohint->po_help_hook;
+      flags |= MU_PARSEOPT_HELP_HOOK;
+    }
+  
   if (setup->prog_doc_hook)
     {
       po.po_prog_doc_hook = prog_doc_hook;
@@ -610,7 +626,22 @@ mu_cli_ext (int argc, char **argv,
     po.po_version_hook = pohint->po_version_hook;
   if (flags & MU_PARSEOPT_NEGATION)
     po.po_negation = pohint->po_negation;
-
+  if (flags & MU_PARSEOPT_PROG_NAME)
+    po.po_prog_name = pohint->po_prog_name;
+  if (flags & MU_PARSEOPT_SPECIAL_ARGS)
+    po.po_special_args = pohint->po_special_args;
+  
+  if (setup->ex_usage)
+    {
+      po.po_exit_error = setup->ex_usage;
+      flags |= MU_PARSEOPT_EXIT_ERROR;
+    }
+  else if (pohint->po_flags & MU_PARSEOPT_EXIT_ERROR)
+    {
+      po.po_exit_error = pohint->po_exit_error;
+      flags |= MU_PARSEOPT_EXIT_ERROR;
+    }
+  
   appd.setup = setup;
   appd.hints = &hints;
   appd.append_tree = NULL;
@@ -618,8 +649,6 @@ mu_cli_ext (int argc, char **argv,
   po.po_data = &appd;
   flags |= MU_PARSEOPT_DATA;
 
-  po.po_exit_error = setup->ex_usage;
-
   mu_opool_create (&pool, MU_OPOOL_ENOMEMABRT);
   optv = init_options (pool, capa, setup, &hints, &com_list);
 
