@@ -1,5 +1,5 @@
 /* Simple inline comment filter for GNU Mailutils.
-   Copyright (C) 2010-2019 Free Software Foundation, Inc.
+   Copyright (C) 2010-2020 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -216,7 +216,10 @@ _ilcmt_decoder (void *xd, enum mu_filter_command cmd,
 		  if (!(pd->flags & ILCMT_SQUEEZE_WS))
 		    {
 		      if (ilcmt_save (pd, *iptr))
-			return mu_filter_failure;
+			{
+			  iobuf->errcode = errno;
+			  return mu_filter_failure;
+			}
 		    }
 		  iptr++;
 		}
@@ -290,7 +293,10 @@ _ilcmt_decoder (void *xd, enum mu_filter_command cmd,
 	      if (!(pd->flags & ILCMT_SQUEEZE_WS))
 		{
 		  if (ilcmt_save (pd, *iptr))
-		    return mu_filter_failure;
+		    {
+		      iobuf->errcode = errno;
+		      return mu_filter_failure;
+		    }
 		}
 	      iptr++;
 	    }
