@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2001, 2006-2007, 2009-2012, 2014-2016 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2020 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -24,28 +23,15 @@ struct mu_message
 {
   mu_message_t msg;       /* Message itself */
   SCM mbox;               /* Mailbox it belongs to */
-  int needs_destroy;      /* Set during mark phase if the message needs
-			     explicit destroying */
 };
 
 /* SMOB functions: */
-
-static SCM
-mu_scm_message_mark (SCM message_smob)
-{
-  struct mu_message *mum = (struct mu_message *) SCM_CDR (message_smob);
-  if (mu_message_get_owner (mum->msg) == NULL)
-    mum->needs_destroy = 1;
-  return mum->mbox;
-}
-
-static scm_sizet
+static size_t
 mu_scm_message_free (SCM message_smob)
 {
   struct mu_message *mum = (struct mu_message *) SCM_CDR (message_smob);
-  if (mum->needs_destroy)
-    mu_message_destroy (&mum->msg, NULL);
-  free (mum);
+  if (mum->mbox == NULL)
+    mu_message_destroy (&mum->msg, mu_message_get_owner (mum->msg));
   return 0;
 }
 
@@ -54,7 +40,7 @@ _get_envelope_sender (mu_envelope_t env)
 {
   mu_address_t addr;
   const char *buffer;
-  char *ptr;
+  char *ptr = NULL;
   
   if (mu_envelope_sget_sender (env, &buffer)
       || mu_address_create (&addr, buffer))
@@ -65,7 +51,7 @@ _get_envelope_sender (mu_envelope_t env)
       mu_address_destroy (&addr);
       return NULL;
     }
-
+  
   mu_address_destroy (&addr);
   return ptr;
 }
@@ -76,7 +62,6 @@ mu_scm_message_print (SCM message_smob, SCM port, scm_print_state * pstate)
   struct mu_message *mum = (struct mu_message *) SCM_CDR (message_smob);
   mu_envelope_t env = NULL;
   const char *buffer;
-  const char *p;
   size_t m_size = 0, m_lines = 0;
   struct tm tm;
   struct mu_timezone tz;
@@ -93,18 +78,22 @@ mu_scm_message_print (SCM message_smob, SCM port, scm_print_state * pstate)
     }
   else
     {
+      char *p;
+      char const *s;
+      mu_attribute_t attr;
+      
       p = _get_envelope_sender (env);
       scm_puts ("\"", port);
       if (p)
 	{
 	  scm_puts (p, port);
-	  free ((void *) p);
+	  free (p);
 	}
       else
 	scm_puts ("UNKNOWN", port);
       
-      if (mu_envelope_sget_date (env, &p) == 0
-          && mu_scan_datetime (p, MU_DATETIME_FROM, &tm, &tz, NULL) == 0)
+      if (mu_envelope_sget_date (env, &s) == 0
+          && mu_scan_datetime (s, MU_DATETIME_FROM, &tm, &tz, NULL) == 0)
 	{
 	  strftime (datebuf, sizeof (datebuf), "%a %b %e %H:%M", &tm);
 	  buffer = datebuf;
@@ -117,10 +106,25 @@ mu_scm_message_print (SCM message_smob, SCM port, scm_print_state * pstate)
       
       mu_message_size (mum->msg, &m_size);
       mu_message_lines (mum->msg, &m_lines);
-      
-      snprintf (datebuf, sizeof (datebuf), "%3lu %-5lu",
-		(unsigned long) m_lines, (unsigned long) m_size);
-      scm_puts (datebuf, port);
+
+      scm_intprint (m_lines, 10, port);
+      scm_putc (' ', port);
+      scm_intprint (m_size, 10, port);
+
+      if (mu_message_get_attribute (mum->msg, &attr) == 0)
+	{
+	  char abuf[MU_STATUS_BUF_SIZE];
+	  if (mu_attribute_to_string (attr, abuf, sizeof(abuf), NULL) == 0)
+	    {
+	      if (abuf[0])
+		{
+	      	  scm_putc (' ', port);
+		  scm_puts (abuf, port);
+		}
+	    }
+	  else
+	    scm_puts (" E", port);	    
+	}
     }
   scm_puts (">", port);
   return 1;
@@ -136,7 +140,6 @@ mu_scm_message_create (SCM owner, mu_message_t msg)
   mum = scm_gc_malloc (sizeof (struct mu_message), "message");
   mum->msg = msg;
   mum->mbox = owner;
-  mum->needs_destroy = 0;
   SCM_RETURN_NEWSMOB (message_tag, mum);
 }
 
@@ -146,17 +149,13 @@ mu_scm_message_add_owner (SCM MESG, SCM owner)
   struct mu_message *mum = (struct mu_message *) SCM_CDR (MESG);
   SCM cell;
 
-  if (scm_is_bool (mum->mbox))
-    {
-      mum->mbox = owner;
-      return;
-    }
-  
-  if (scm_is_pair (mum->mbox))
-    cell = scm_cons (owner, mum->mbox);
+  if (mum->mbox == NULL)
+    mum->mbox = scm_cons (owner, SCM_EOL);
   else
-    cell = scm_cons (owner, scm_cons (mum->mbox, SCM_EOL));
-  mum->mbox = cell;
+    {
+      cell = scm_cons (owner, mum->mbox);
+      mum->mbox = cell;
+    }
 }
 
 mu_message_t
@@ -289,10 +288,13 @@ SCM_DEFINE_PUBLIC (scm_mu_message_set_header, "mu-message-set-header", 3, 1, 0,
 
   hdr_c = scm_to_locale_string (header);
   val_c = scm_to_locale_string (value);
-  status = mu_header_set_value (hdr, hdr_c, val_c, repl);
+  if (repl)
+    status = mu_header_set_value (hdr, hdr_c, val_c, repl);
+  else
+    status = mu_header_append (hdr, hdr_c, val_c);
   free (hdr_c);
   free (val_c);
-  
+
   if (status)
     mu_scm_error (FUNC_NAME, status,
 		  "Cannot set header \"~A: ~A\" in message ~A",
@@ -304,7 +306,7 @@ SCM_DEFINE_PUBLIC (scm_mu_message_set_header, "mu-message-set-header", 3, 1, 0,
 
 SCM_DEFINE_PUBLIC (scm_mu_message_get_size, "mu-message-get-size", 1, 0, 0,
 		   (SCM mesg),
-		   "Returns size of the message @var{mesg}\n.")
+		   "Returns size of the message @var{mesg}.\n")
 #define FUNC_NAME s_scm_mu_message_get_size
 {
   mu_message_t msg;
@@ -361,7 +363,8 @@ filltime (struct tm *bd_time, int zoff, const char *zname)
 
 SCM_DEFINE_PUBLIC (scm_mu_message_get_envelope, "mu-message-get-envelope", 1, 0, 0,
 		   (SCM mesg),
-		   "Returns envelope date of the message @var{mesg}.\n")
+"Returns envelope of the message @var{mesg}.\n"
+"The value returned is the envelope line without the \"From \" prefix.\n")
 #define FUNC_NAME s_scm_mu_message_get_envelope
 {
   mu_message_t msg;
@@ -432,7 +435,6 @@ SCM_DEFINE_PUBLIC (scm_mu_message_get_sender, "mu-message-get-sender", 1, 0, 0,
   mu_message_t msg;
   mu_envelope_t env = NULL;
   int status;
-  SCM ret;
   
   SCM_ASSERT (mu_scm_is_message (mesg), mesg, SCM_ARG1, FUNC_NAME);
   msg = mu_scm_message_get (mesg);
@@ -440,14 +442,18 @@ SCM_DEFINE_PUBLIC (scm_mu_message_get_sender, "mu-message-get-sender", 1, 0, 0,
   if (status == 0)
     {
       char *p = _get_envelope_sender (env);
-      ret = scm_from_locale_string (p);
-      free (p);
+      if (p)
+	{
+	  SCM ret = scm_from_locale_string (p);
+	  free (p);
+	  return ret;
+	}
     }
-  else
-    mu_scm_error (FUNC_NAME, status,
-		  "Cannot get envelope of message ~A",
-		  scm_list_1 (mesg));
-  return ret;
+
+  mu_scm_error (FUNC_NAME, status,
+		"Cannot get envelope of message ~A",
+		scm_list_1 (mesg));
+  return SCM_UNDEFINED;
 }
 #undef FUNC_NAME
 
@@ -495,6 +501,25 @@ SCM_DEFINE_PUBLIC (scm_mu_message_get_header, "mu-message-get-header", 2, 0, 0,
 }
 #undef FUNC_NAME
 
+static int
+string_is_member (char const *needle, SCM array)
+{
+  int found = 0;
+  
+  while (!(found || scm_is_null (array)))
+    {
+      SCM car = scm_car (array);
+      if (scm_is_string (car))
+	{
+	  char *s = scm_to_locale_string (car);
+	  found = mu_c_strcasecmp (needle, s) == 0;
+	  free (s);
+	}
+      array = scm_cdr (array);
+    }
+  return found;
+}
+
 SCM_DEFINE_PUBLIC (scm_mu_message_get_header_fields, "mu-message-get-header-fields", 1, 1, 0,
 		   (SCM mesg, SCM headers),
 "Returns list of headers in the message @var{mesg}. optional argument\n" 
@@ -534,8 +559,7 @@ SCM_DEFINE_PUBLIC (scm_mu_message_get_header_fields, "mu-message-get-header-fiel
 		      "Cannot get header field ~A, message ~A",
 		      scm_list_2 (scm_from_size_t (i), mesg));
 
-      if (!scm_is_null (headers) &&
-	  scm_member (headers, scm_from_locale_string (name)) == SCM_BOOL_F)
+      if (!scm_is_null (headers) && !string_is_member (name, headers))
 	continue;
       status = mu_header_aget_field_value (hdr, i, &value);
       if (status)
@@ -602,7 +626,10 @@ SCM_DEFINE_PUBLIC (scm_mu_message_set_header_fields, "mu-message-set-header-fiel
       SCM_ASSERT (scm_is_string (cdr), cdr, SCM_ARGn, FUNC_NAME);
       hdr_c = scm_to_locale_string (car);
       val_c = scm_to_locale_string (cdr);
-      status = mu_header_set_value (hdr, hdr_c, val_c, repl);
+      if (repl)
+	status = mu_header_set_value (hdr, hdr_c, val_c, repl);
+      else
+	status = mu_header_append (hdr, hdr_c, val_c);
       free (hdr_c);
       free (val_c);
       if (status)
@@ -869,13 +896,13 @@ SCM_DEFINE_PUBLIC (scm_mu_message_set_user_flag, "mu-message-set-user-flag", 2,
 
 SCM_DEFINE_PUBLIC (scm_mu_message_get_port, "mu-message-get-port", 2, 1, 0,
 		   (SCM mesg, SCM mode, SCM full),
+"@anchor{mu-message-get-port}\n"		   
 "Returns a port associated with the message @var{mesg}. The @var{mode} is a\n"
 "string defining operation mode of the stream. It may contain any of the\n"
 "two characters: @samp{r} for reading, @samp{w} for writing.\n"
-"If optional argument @var{full} is specified, it should be a boolean value.\n"
-"If it is @samp{#t} then the returned port will allow access to any\n"
-"part of the message (including headers). If it is @code{#f} then the port\n"
-"accesses only the message body (the default).\n")
+"If optional boolean argument @var{full} is @samp{#t} then the returned port\n"
+"will allow access to any part of the message (including headers). Otherwise\n"
+"the port accesses only the message body (the default).\n")
 #define FUNC_NAME s_scm_mu_message_get_port
 {
   mu_message_t msg;
@@ -916,7 +943,7 @@ SCM_DEFINE_PUBLIC (scm_mu_message_get_port, "mu-message-get-port", 2, 1, 0,
     }
 
   str = scm_to_locale_string (mode);
-  ret = mu_port_make_from_stream (mesg, stream, scm_mode_bits (str));
+  ret = mu_port_make_from_stream (stream, scm_mode_bits (str));
   free (str);
   return ret;
 }
@@ -1012,7 +1039,7 @@ SCM_DEFINE_PUBLIC (scm_mu_message_get_part, "mu-message-get-part", 2, 0, 0,
 SCM_DEFINE_PUBLIC (scm_mu_message_send, "mu-message-send", 1, 3, 0,
 		   (SCM mesg, SCM mailer, SCM from, SCM to),
 "Sends message @var{mesg}. Optional @var{mailer} overrides default mailer\n"
-"settings. Optional @var{from} and @var{to} give sender and recever\n"
+"settings. Optional @var{from} and @var{to} give sender and receiver\n"
 "addresses, respectively.\n")
 #define FUNC_NAME s_scm_mu_message_send
 {
@@ -1111,13 +1138,37 @@ SCM_DEFINE_PUBLIC (scm_mu_message_get_uid, "mu-message-get-uid", 1, 0, 0,
 }
 #undef FUNC_NAME
 
+SCM_DEFINE_PUBLIC (scm_mu_message_from_port, "mu-message-from-port", 1, 0, 0,
+		   (SCM port),
+"Reads one message from @var{port} and returns it.\n")
+#define FUNC_NAME s_scm_mu_message_from_port
+{
+  mu_stream_t str;
+  int rc;
+  mu_message_t msg;
+  
+  SCM_ASSERT (SCM_PORTP (port), port, SCM_ARG1, FUNC_NAME);
+  rc = mu_scm_port_stream_create (&str, port);
+  if (rc)
+    mu_scm_error (FUNC_NAME, rc,
+		  "Cannot create mu_stream_t from ~A",
+		  scm_list_1 (port));
+  rc = mu_stream_to_message (str, &msg);
+  mu_stream_destroy (&str);
+  if (rc)
+    mu_scm_error (FUNC_NAME, rc,
+		  "Failed to get message from ~A",
+		  scm_list_1 (port));
+  return mu_scm_message_create (NULL, msg);
+}
+#undef FUNC_NAME
+
 /* Initialize the module */
 
 void
 mu_scm_message_init ()
 {
   message_tag = scm_make_smob_type ("message", sizeof (struct mu_message));
-  scm_set_smob_mark (message_tag, mu_scm_message_mark);
   scm_set_smob_free (message_tag, mu_scm_message_free);
   scm_set_smob_print (message_tag, mu_scm_message_print);
 
