@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2001, 2006-2007, 2010-2012, 2014-2016 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2020 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -27,21 +26,12 @@ struct mu_mime
 };
 
 /* SMOB functions: */
-
-static SCM
-mu_scm_mime_mark (SCM mime_smob)
-{
-  struct mu_mime *mum = (struct mu_mime *) SCM_CDR (mime_smob);
-  return mum->owner;
-}
-
-static scm_sizet
+static size_t
 mu_scm_mime_free (SCM mime_smob)
 {
   struct mu_mime *mum = (struct mu_mime *) SCM_CDR (mime_smob);
   mu_mime_destroy (&mum->mime);
-  free (mum);
-  return sizeof (struct mu_mime);
+  return 0;
 }
 
 static int
@@ -110,7 +100,7 @@ SCM_DEFINE_PUBLIC (scm_mu_mime_create, "mu-mime-create", 0, 2, 0,
   int fl;
   int status;
   
-  if (scm_is_bool (flags))
+  if (SCM_UNBNDP (flags) || scm_is_bool (flags))
     {
       /*if (flags == SCM_BOOL_F)*/
       fl = 0;
@@ -241,7 +231,6 @@ void
 mu_scm_mime_init ()
 {
   mime_tag = scm_make_smob_type ("mime", sizeof (struct mu_mime));
-  scm_set_smob_mark (mime_tag, mu_scm_mime_mark);
   scm_set_smob_free (mime_tag, mu_scm_mime_free);
   scm_set_smob_print (mime_tag, mu_scm_mime_print);
 
