@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001-2002, 2007, 2010-2012, 2014-2016 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2020 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -96,7 +95,7 @@ mail_if (int argc, char **argv)
       return 1;
     }
 
-  if (mailvar_get (&mode, "mode", mailvar_type_string, 1))
+  if (mailvar_get (&mode, mailvar_name_mode, mailvar_type_string, 1))
     exit (EXIT_FAILURE);
 
   if (if_cond() == 0)
