@@ -236,9 +236,7 @@ typedef struct message_set msgset_t;
 struct message_set
 {
   msgset_t *next;       /* Link to the next message set */
-  unsigned int npart;   /* Number of parts in this set */
-  size_t *msg_part;     /* Array of part numbers: msg_part[0] is the 
-                           message number */
+  mu_coord_t crd;
 };
 
 typedef int (*msg_handler_t) (msgset_t *mp, mu_message_t mesg, void *data);
@@ -402,6 +400,26 @@ int msgset_parse (const int argc, char **argv, int flags, msgset_t **mset);
 int msgset_member (msgset_t *set, size_t n);
 msgset_t *msgset_negate (msgset_t *set);
 size_t msgset_count (msgset_t *set);
+static inline size_t
+msgset_length (msgset_t *set)
+{
+  return mu_coord_length (set->crd);
+}
+static inline size_t
+msgset_msgno (msgset_t *set)
+{
+  return set->crd[1];
+}
+static inline char *
+msgset_part_str (const msgset_t *set, size_t n)
+{
+  return mu_coord_part_string (set->crd, n);
+}
+static inline char *
+msgset_str (const msgset_t *set)
+{
+  return mu_coord_string (set->crd);
+}
 
 #define MDHINT_SELECTED_HEADERS 0x1
 
@@ -490,6 +508,9 @@ void util_get_hdr_value (mu_header_t hdr, const char *name, char **value);
 int util_merge_addresses (char **addr_str, const char *value);
 int util_header_expand (mu_header_t *hdr);
 int util_get_message (mu_mailbox_t mbox, size_t msgno, mu_message_t *msg);
+int util_get_message_part (mu_mailbox_t mbox, msgset_t *msgset,
+			   mu_message_t *ret_msg);
+
 void util_cache_command (mu_list_t *list, const char *fmt, ...)
   MU_PRINTFLIKE(2,3);
 void util_run_cached_commands (mu_list_t *list);
@@ -503,7 +524,8 @@ const char *util_url_to_string (mu_url_t url);
 
 mu_stream_t open_pager (size_t lines);
 
-void format_msgset (mu_stream_t str, const msgset_t *msgset, size_t *count);
+int print_message_body (mu_message_t msg, mu_stream_t out, size_t *stat);
+
 char *sender_string (mu_message_t msg);
 
 int is_address_field (const char *name);
@@ -535,7 +557,7 @@ mu_address_t get_sender_address (mu_message_t msg);
 void compose_init (compose_env_t *env);
 int compose_header_set (compose_env_t *env, const char *name,
 		        const char *value, int replace);
-char *compose_header_get (compose_env_t *env, char *name, char *defval);
+char const *compose_header_get (compose_env_t *env, char *name, char *defval);
 void compose_destroy (compose_env_t *env);
 
 #ifndef HAVE_READLINE_READLINE_H
